----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:15:55 10/26/2009 
-- Design Name: 
-- Module Name:    TOPFILE - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library UNISIM;
use UNISIM.VComponents.all;


entity S2P_1laneDATA is
Port ( CLOCK_IN_P : in  STD_LOGIC;       --LVDS Takt
       CLOCK_IN_N : in  STD_LOGIC;       --LVDS Takt
		 RESET : in  STD_LOGIC;            --Schieberegister Reset
		 RESET_DCM : in STD_LOGIC;         --DCM Reset
		 FULL_ris : out  std_logic;		  --schieberegister mit 8bit geladen
--		 FULL_fal : out  std_logic;		  --schieberegister mit 8bit geladen
       DATA_IN_P : in  STD_LOGIC;        --serielle Daten Eingang
		 DATA_IN_N : in  STD_LOGIC;        --serielle Daten Eingang
		 DATA_OUT_ris : out STD_LOGIC_VECTOR (7 downto 0); --parallele Daten Ausgang steigende Flanke
--		 DATA_OUT_fal : out STD_LOGIC_VECTOR (7 downto 0); --parallele Daten Ausgang fallende Flanke
		 SET_IDDR2 : in  STD_LOGIC
      );
end S2P_1laneDATA;

architecture Behavioral of S2P_1laneDATA is

COMPONENT DIFF_CLOCK_IN
	PORT(
		CLKIN_N_IN : IN std_logic;
		CLKIN_P_IN : IN std_logic;
		RST_IN : IN std_logic;          
		CLKIN_IBUFGDS_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		CLK180_OUT : OUT std_logic;
		LOCKED_OUT : OUT std_logic
		);
	END COMPONENT;

COMPONENT DIFF_DATA_IN
	PORT(
		DATA_N : IN std_logic;
		DATA_P : IN std_logic;          
		DATA_OUT : OUT std_logic
		);
	END COMPONENT;

COMPONENT schiebereg8bit
	PORT(
		CLOCK_SR : IN std_logic;
		RESET_SR : IN std_logic;
		DATA_IN_SR : IN std_logic;          
		FULL : OUT std_logic;
		DATA_OUT_SR : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;

signal CLOCK_IN_DCM0 : STD_LOGIC;
signal CLOCK_IN_DCM180 : STD_LOGIC;
signal DCM_DIFF_LOCK : STD_LOGIC;
signal CLOCK_IN : STD_LOGIC;
signal DATA_IN : STD_LOGIC;
signal DATA_IN_ris : STD_LOGIC;
signal DATA_IN_fal : STD_LOGIC;
signal DATA_READ_EN_ris : STD_LOGIC;
signal DATA_READ_EN_fal : STD_LOGIC;
signal DATA_OUT_INT_ris : STD_LOGIC_VECTOR (7 downto 0);
signal DATA_OUT_INT_fal : STD_LOGIC_VECTOR (7 downto 0);

constant IDDR2_CE : STD_LOGIC := '1';
	
begin

Inst_DIFF_CLOCK_IN: DIFF_CLOCK_IN PORT MAP(
		CLKIN_N_IN => CLOCK_IN_N,
		CLKIN_P_IN => CLOCK_IN_P,
		RST_IN => RESET_DCM,
		CLKIN_IBUFGDS_OUT => CLOCK_IN,
		CLK0_OUT => CLOCK_IN_DCM0,
		CLK180_OUT => CLOCK_IN_DCM180,
		LOCKED_OUT => DCM_DIFF_LOCK
	);

Inst_DIFF_DATA_IN: DIFF_DATA_IN PORT MAP(
		DATA_OUT => DATA_IN,
		DATA_N => DATA_IN_N,
		DATA_P => DATA_IN_P
	);

IDDR2_inst : IDDR2
generic map (
   DDR_ALIGNMENT => "NONE", -- Sets output alignment 
                            -- to "NONE", "C0" or "C1"
   INIT_Q0 => '0', -- Sets initial state of the Q0  
                   --   output to ?0? or ?1?
   INIT_Q1 => '0', -- Sets initial state of the Q1 
                   --   output to ?0? or ?1?
   SRTYPE => "SYNC") -- Specifies "SYNC" or "ASYNC" 
                      --   set/reset
port map (
   Q0 => DATA_IN_ris, -- 1-bit output captured with C0 clock
   Q1 => DATA_IN_fal, -- 1-bit output captured with C1 clock
   C0 => CLOCK_IN_DCM0, -- 1-bit clock input
   C1 => CLOCK_IN_DCM180, -- 1-bit clock input
   CE => IDDR2_CE, -- 1-bit clock enable input
   D => DATA_IN,   -- 1-bit DDR data input
   R => RESET,   -- 1-bit reset input
   S => SET_IDDR2    -- 1-bit set input
);


Inst_schiebereg8bit1: schiebereg8bit PORT MAP(
		CLOCK_SR => CLOCK_IN_DCM0,
		RESET_SR => RESET,
		DATA_IN_SR => DATA_IN_ris,
		FULL => DATA_READ_EN_ris,
		DATA_OUT_SR => DATA_OUT_INT_ris
	);

--Inst_schiebereg8bit2: schiebereg8bit PORT MAP(
--		CLOCK_SR => CLOCK_IN_DCM0,
--		RESET_SR => RESET,
--		DATA_IN_SR => DATA_IN_fal,
--		FULL => DATA_READ_EN_fal,
--		DATA_OUT_SR => DATA_OUT_INT_fal
--	);
	
READ_EN : process (RESET, DATA_READ_EN_ris, DATA_READ_EN_fal, DATA_OUT_INT_ris, DATA_OUT_INT_fal)
begin
		if (RESET = '1') then
			DATA_OUT_ris <= (others => '0');
		elsif (DATA_READ_EN_ris = '1') then
			DATA_OUT_ris <= DATA_OUT_INT_ris;
		else
			DATA_OUT_ris <= (others => '0');
		end if;
		
--		if (RESET = '1') then
--			DATA_OUT_fal <= (others => '0');
--		elsif (DATA_READ_EN_fal = '1') then
--			DATA_OUT_fal <= DATA_OUT_INT_fal;
--		else
--			DATA_OUT_fal <= (others => '0');
--		end if;
		
end process;

FULL_ris <= DATA_READ_EN_ris;
--FULL_fal <= DATA_READ_EN_fal;


end Behavioral;

